from flask import Flask, render_template, request, jsonify
import random
import re
import json
from thefuzz import fuzz

app = Flask(__name__)

with open("intents.json") as file:
    intents = json.load(file)

class Chatbot:
    def __init__(self, intents):
        self.intents = intents['intents']
        self.default_responses = [
            "Sorry, I don't understand that. I can assist with general location, hours, and services information.",
            "I couldn't quite catch that. Try asking about locations, hours, or services.",
            "I'm not sure about that one. Feel free to ask me about campus buildings, hours, or services.",
        ]
        self.menu_link = "https://www.mado.menu/food-menu/"

    def preprocess_text(self, text):
        # Normalize text by converting to lowercase and removing punctuation
        text = text.lower()
        text = re.sub(r'[^\w\s]', '', text)
        return text

    def find_match_token_set_ratio(self, user_input, threshold=75):
        user_input = self.preprocess_text(user_input)
        best_match = 0
        match = None

        for intent in self.intents:
            for pattern in intent['patterns']:
                pattern = self.preprocess_text(pattern)
                similarity = fuzz.token_set_ratio(user_input, pattern)
                if similarity >= threshold and similarity > best_match:
                    best_match = similarity
                    match = intent['tag']

        return match

    def get_response(self, user_input, threshold=70):
        tag = self.find_match_token_set_ratio(user_input, threshold)

        if tag:
            for intent in self.intents:
                if intent['tag'] == tag:
                    responses = intent['responses']
                    response = random.choice(responses)
                    # If the response contains a placeholder for the menu link, replace it with the actual link
                    if '[Menu Link]' in response:
                        response = response.replace('[Menu Link]', self.menu_link)
                    return response

        return random.choice(self.default_responses)

chatbot = Chatbot(intents)

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/', methods=['POST'])
def chat():
    user_input = request.form['prompt']
    response = chatbot.get_response(user_input)
    return jsonify(response)

if __name__ == "__main__":
    app.run(debug=True)